@@ -1,7 +1,6 @@
-/* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2021 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2018 Roy Marples <roy@marples.name>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,7 +38,6 @@
 
 #include "../common.h"
 #include "../dev.h"
-#include "../if.h"
 #include "../logerr.h"
 
 static const char udev_name[] = "udev";
@@ -56,7 +54,7 @@ udev_listening(void)
 }
 
 static int
-udev_initialised(const char *ifname)
+udev_initialized(const char *ifname)
 {
 	struct udev_device *device;
 	int r;
@@ -121,14 +119,8 @@ udev_stop(void)
 static int
 udev_start(void)
 {
-	char netns[PATH_MAX];
 	int fd;
 
-	if (if_getnetworknamespace(netns, sizeof(netns)) != NULL) {
-		logdebugx("udev does not work in a network namespace");
-		return -1;
-	}
-
 	if (udev) {
 		logerrx("udev: already started");
 		return -1;
@@ -174,7 +166,7 @@ dev_init(struct dev *dev, const struct dev_dhcpcd *dev_dhcpcd)
 {
 
 	dev->name = udev_name;
-	dev->initialised = udev_initialised;
+	dev->initialized = udev_initialized;
 	dev->listening = udev_listening;
 	dev->handle_device = udev_handle_device;
 	dev->stop = udev_stop;
