@@ -609,7 +609,7 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 			rta = RTA_NEXT(rta, len);
 		}
 		ipv4_handleifa(ctx, nlm->nlmsg_type, NULL, ifp->name,
-		    &addr, &net, &brd, ifa->ifa_flags);
+		    &addr, &net, &brd, ifa->ifa_flags, (pid_t)nlm->nlmsg_pid);
 		break;
 #endif
 #ifdef INET6
@@ -625,7 +625,8 @@ link_addr(struct dhcpcd_ctx *ctx, struct interface *ifp, struct nlmsghdr *nlm)
 			rta = RTA_NEXT(rta, len);
 		}
 		ipv6_handleifa(ctx, nlm->nlmsg_type, NULL, ifp->name,
-		    &addr6, ifa->ifa_prefixlen, ifa->ifa_flags);
+		    &addr6, ifa->ifa_prefixlen, ifa->ifa_flags,
+		    (pid_t)nlm->nlmsg_pid);
 		break;
 #endif
 	}
@@ -1680,36 +1681,31 @@ if_disable_autolinklocal(struct dhcpcd_ctx *ctx, unsigned int ifindex)
 
 static const char *prefix = "/proc/sys/net/ipv6/conf";
 
-int
-if_checkipv6(struct dhcpcd_ctx *ctx, const struct interface *ifp)
+void
+if_setup_inet6(const struct interface *ifp)
 {
-	const char *ifname;
 	int ra;
 	char path[256];
 
-	if (ifp == NULL)
-		ifname = "all";
-	else if (!(ctx->options & DHCPCD_TEST)) {
-		if (if_disable_autolinklocal(ctx, ifp->index) == -1)
-			logdebug("%s: if_disable_autolinklocal",
-			    ifp->name);
-	}
-	if (ifp)
-		ifname = ifp->name;
+	/* The kernel cannot make stable private addresses. */
+	if (if_disable_autolinklocal(ifp->ctx, ifp->index) == -1)
+		logdebug("%s: if_disable_autolinklocal", ifp->name);
 
-	snprintf(path, sizeof(path), "%s/%s/autoconf", prefix, ifname);
+	/*
+	 * If not doing autoconf, don't disable the kernel from doing it.
+	 * If we need to, we should have another option actively disable it.
+	 */
+	if (!(ifp->options->options & DHCPCD_IPV6RS))
+		return;
+
+	snprintf(path, sizeof(path), "%s/%s/autoconf", prefix, ifp->name);
 	ra = check_proc_int(path);
-	if (ra != 1) {
-		if (ctx->options & DHCPCD_TEST)
-			logwarnx("%s: IPv6 kernel autoconf disabled", ifname);
-	} else if (ra != -1 && !(ctx->options & DHCPCD_TEST)) {
-		if (write_path(path, "0") == -1) {
+	if (ra != 1 && ra != -1) {
+		if (write_path(path, "0") == -1)
 			logerr("%s: %s", __func__, path);
-			return -1;
-		}
 	}
 
-	snprintf(path, sizeof(path), "%s/%s/accept_ra", prefix, ifname);
+	snprintf(path, sizeof(path), "%s/%s/accept_ra", prefix, ifp->name);
 	ra = check_proc_int(path);
 	if (ra == -1) {
 		logfunc_t *logfunc = errno == ENOENT? logdebug : logwarn;
@@ -1717,16 +1713,10 @@ if_checkipv6(struct dhcpcd_ctx *ctx, const struct interface *ifp)
 		/* The sysctl probably doesn't exist, but this isn't an
 		 * error as such so just log it and continue */
 		logfunc("%s", path);
-	} else if (ra != 0 && !(ctx->options & DHCPCD_TEST)) {
-		logdebugx("%s: disabling kernel IPv6 RA support", ifname);
-		if (write_path(path, "0") == -1) {
+	} else if (ra != 0) {
+		if (write_path(path, "0") == -1)
 			logerr("%s: %s", __func__, path);
-			return ra;
-		}
-		return 0;
 	}
-
-	return ra;
 }
 
 #ifdef IPV6_MANAGETEMPADDR
