@@ -377,25 +377,26 @@ static void
 ipv6nd_reachable(struct ra *rap, int flags)
 {
 
+	if (rap->lifetime == 0)
+		return;
+
 	if (flags & IPV6ND_REACHABLE) {
-		if (rap->lifetime && rap->expired) {
-			loginfox("%s: %s is reachable again",
-			    rap->iface->name, rap->sfrom);
-			rap->expired = 0;
-			rt_build(rap->iface->ctx, AF_INET6);
-			/* XXX Not really an RA */
-			script_runreason(rap->iface, "ROUTERADVERT");
-		}
+		if (rap->expired == 0)
+			return;
+		loginfox("%s: %s is reachable again",
+		    rap->iface->name, rap->sfrom);
+		rap->expired = 0;
 	} else {
-		if (rap->lifetime && !rap->expired) {
-			logwarnx("%s: %s is unreachable, expiring it",
-			    rap->iface->name, rap->sfrom);
-			rap->expired = 1;
-			rt_build(rap->iface->ctx, AF_INET6);
-			/* XXX Not really an RA */
-			script_runreason(rap->iface, "ROUTERADVERT");
-		}
+		if (rap->expired != 0)
+			return;
+		logwarnx("%s: %s is unreachable, expiring it",
+		    rap->iface->name, rap->sfrom);
+		rap->expired = 1;
 	}
+
+	rt_build(rap->iface->ctx, AF_INET6);
+	/* XXX Not really an RA */
+	script_runreason(rap->iface, "ROUTERADVERT");
 }
 
 void
@@ -869,7 +870,9 @@ ipv6nd_handlera(struct dhcpcd_ctx *ctx, struct interface *ifp,
 		rap->expired = 0;
 	rap->hasdns = 0;
 
-	ipv6_settempstale(ifp);
+#ifdef IPV6_AF_TEMPORARY
+	ipv6_markaddrsstale(ifp, IPV6_AF_TEMPORARY);
+#endif
 	TAILQ_FOREACH(ap, &rap->addrs, next) {
 		ap->flags |= IPV6_AF_STALE;
 	}
@@ -1291,7 +1294,7 @@ ipv6nd_env(char **env, const char *prefix, const struct interface *ifp)
 }
 
 void
-ipv6nd_handleifa(int cmd, struct ipv6_addr *addr)
+ipv6nd_handleifa(int cmd, struct ipv6_addr *addr, pid_t pid)
 {
 	struct ra *rap;
 
@@ -1303,7 +1306,7 @@ ipv6nd_handleifa(int cmd, struct ipv6_addr *addr)
 	TAILQ_FOREACH(rap, addr->iface->ctx->ra_routers, next) {
 		if (rap->iface != addr->iface)
 			continue;
-		ipv6_handleifa_addrs(cmd, &rap->addrs, addr);
+		ipv6_handleifa_addrs(cmd, &rap->addrs, addr, pid);
 	}
 }
 
@@ -1313,7 +1316,7 @@ ipv6nd_expirera(void *arg)
 	struct interface *ifp;
 	struct ra *rap, *ran;
 	struct timespec now, lt, expire, next;
-	uint8_t expired, valid, validone;
+	uint8_t expired, anyvalid, valid, validone;
 	struct ipv6_addr *ia;
 
 	ifp = arg;
@@ -1321,11 +1324,11 @@ ipv6nd_expirera(void *arg)
 	expired = 0;
 	timespecclear(&next);
 
-	validone = 0;
+	anyvalid = 0;
 	TAILQ_FOREACH_SAFE(rap, ifp->ctx->ra_routers, next, ran) {
 		if (rap->iface != ifp)
 			continue;
-		valid = 0;
+		valid = validone = 0;
 		if (rap->lifetime) {
 			lt.tv_sec = (time_t)rap->lifetime;
 			lt.tv_nsec = 0;
@@ -1351,9 +1354,12 @@ ipv6nd_expirera(void *arg)
 		 * the kernel can expire, so we need to handle it ourself.
 		 * Also, some OS don't support address lifetimes (Solaris). */
 		TAILQ_FOREACH(ia, &rap->addrs, next) {
-			if (ia->prefix_vltime == ND6_INFINITE_LIFETIME ||
-			    ia->prefix_vltime == 0)
+			if (ia->prefix_vltime == 0)
 				continue;
+			if (ia->prefix_vltime == ND6_INFINITE_LIFETIME) {
+				validone = 1;
+				continue;
+			}
 			lt.tv_sec = (time_t)ia->prefix_vltime;
 			lt.tv_nsec = 0;
 			timespecadd(&ia->acquired, &lt, &expire);
@@ -1375,6 +1381,7 @@ ipv6nd_expirera(void *arg)
 				if (!timespecisset(&next) ||
 				    timespeccmp(&next, &lt, >))
 					next = lt;
+				validone = 1;
 			}
 		}
 
@@ -1385,10 +1392,10 @@ ipv6nd_expirera(void *arg)
 
 		/* No valid lifetimes are left on the RA, so we might
 		 * as well punt it. */
-		if (!valid && TAILQ_FIRST(&rap->addrs) == NULL)
+		if (!valid && !validone)
 			ipv6nd_free_ra(rap);
 		else
-			validone = 1;
+			anyvalid = 1;
 	}
 
 	if (timespecisset(&next))
@@ -1400,7 +1407,7 @@ ipv6nd_expirera(void *arg)
 	}
 
 	/* No valid routers? Kill any DHCPv6. */
-	if (!validone)
+	if (!anyvalid)
 		dhcp6_dropnondelegates(ifp);
 }
 
@@ -1501,15 +1508,8 @@ ipv6nd_handlena(struct dhcpcd_ctx *ctx, struct interface *ifp,
 		return;
 	}
 
-	if (is_solicited && is_router && rap->lifetime) {
-		if (rap->expired) {
-			rap->expired = 0;
-			loginfox("%s: %s reachable (%s)",
-			    ifp->name, taddr, ctx->sfrom);
-			rt_build(ifp->ctx, AF_INET6);
-			script_runreason(rap->iface, "ROUTERADVERT"); /* XXX */
-		}
-	}
+	if (is_solicited && is_router && rap->lifetime)
+		ipv6nd_reachable(rap, IPV6ND_REACHABLE);
 }
 
 static void
